/*****************************************************************************
 *   Copyright (C) 2011 Christian "Cebra" Brandtner, brandtner@brandtner.net *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License.          *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *                                                                           *
 *****************************************************************************/


/*
 * HAL_HW3_9.c
 *
 *  Created on: 13.07.2011
 *      Author: cebra
 *
 *Hardwareabhängige Definitionen und Prozeduren
 * Hardware Leiterplatte V3.7 Cebra
 * USB,Wi232,LED
 */

#ifndef HAL_HW3_9_C_
#define HAL_HW3_9_C_

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdbool.h>

#include "main.h"

#ifdef HWVERSION3_9

#include "lcd.h"
#include "usart.h"
#include "uart1.h"
#include "display.h"
#include "timer.h"
#include "eeprom.h"
#include "Wi232.h"
#include "twimaster.h"
#include "uart1.h"
#include "bluetooth.h"
#include "error.h"

//#include "HAL_HW3_9.h"



void InitHWPorts(void)	/* Initialisierung der Hardware für die jeweilige Leiterplattenversion */

{

	// enable pull ups for the 4 keys
	PORTA |= (1<<PORTA4)|(1<<PORTA5)|(1<<PORTA6)|(1<<PORTA7);

	DDRD |= (1<<DDD7)|(1<<DDD6); //Wi232, Beleuchtung
	DDRC |= (1<<DDC2)|(1<<DDC3)|(1<<DDC5)|(1<<DDC6)|(1<<DDC7); /* Leds,Summer,Rs232Switch*/
	DDRC &= ~(1 << DDC4);	/* LowBat Eingang*/
	PORTC |= (1<<PORTC4);
	DDRB = 0xFF; 		//alles Ausgänge
	set_V_On();			//Spannung mit T3 halten


	_delay_ms(500);	// Einschaltverzögerung für ENTER-Taste

#ifdef HaraldB
	if ((PINA & (1<<PINA4))) clr_V_On();
#else
	if ((PINA & (1<<PINA7))) clr_V_On();
#endif


	Change_Output(Uart02Wi); // Verbindung zu Wi232 herstellen
	set_LED1();

	set_D_LIGHT();
	Timer0_Init ();
	TIMER1_Init (); // pwm
	LCD_Init (0);
    beeptime = 500;
    BeepMuster = 0x0080;

// _delay_ms(250);

	clr_LED1();
	Display_on = 1;
	USART_Init (UART_BAUD_SELECT(USART_BAUD,F_CPU));
	uart1_init (UART_BAUD_SELECT(USART_BAUD,F_CPU));      /* USB*/
	I2C_Init(1);
	sei ();

	ReadParameter ();
	LCD_Init (1);


	 if (WiIsSet==false) InitWi232(); /* wenn Wi232 nicht initialisiert ist, dann jetzt tun*/
	   else
	   {
		   discoverWi232();			/* Wi232 suchen*/
	       _delay_ms(1000);
	   }


    if (USBBT==0) set_USBOn();
    else
    {
    	set_BTOn();    // USB ein, BT aus
   	    lcd_printpns_at (0, 5, PSTR("BT eingeschaltet"),1);
   	    lcd_printpns_at (0, 6, PSTR("warte ......"),1);
    	_delay_ms(3000);
    }


// Erstmal raus, dauert zulange, Init Funktion einmal nötig im Setupmenü
//	 if (USBBT==1)
//	 {
//		 lcd_printpns_at (0, 4, PSTR("Bluetooth Init...wait"),1);
//		  _delay_ms(2000);	//auf BT warten
//		 if (bt_init())
// 			{
//			 info_pgm(PSTR("BT_Init OK"));
//			 lcd_printpns_at (0, 5, PSTR("Bluetooth OK"),2);
//			 _delay_ms(2000);
// 			}
//		 else
//			 {
//			  lcd_printpns_at (0, 2, PSTR("Bluetooth Fehler"),2);
//			  beeptime = 1000;
//			  BeepMuster = 0x0040;
//			 _delay_ms(2000);
//			 }
//	 }

		uart1_putc('R');

}

void Change_Output(uint8_t UartMode)	/* Schaltet die Rx/Tx Richtungen*/
	{
  		clr_USB2Wi();		//aus
  		clr_USB2FC();		//aus
  		clr_Uart02FC();		//aus
  		clr_Uart02Wi();		//aus
  		switch (UartMode)
  		{
  		case USB2FC:
//  			lcd_printpns_at (0, 5, PSTR("set USB2FC"),0);

  			/* Disable UART receiver and transmitter */
//  			UCSR1B &= ~ ( ( 1 << RXCIE0 ) | ( 1 << RXEN0 ) | ( 1 << TXEN0 ) );

//  			UCSR0B &= ~(1<<UDRIE0);         /* Disable UDRE interrupt */
//  			UCSR0B |= (1<<UDRIE0);          /* Enable UDRE interrupt */

  			UCSR1B &= ~(1<<RXEN1 );
  			UCSR1B &= ~(1<<TXEN1 );
  			UCSR1B &= ~(1<<RXCIE1);



//  			UCSR1B &= ~ _BV(RXCIE1)|(1<<RXEN1)|(1<<TXEN1); // Rx/Tx Disable
  			DDRD &= ~(1 << DDD2);					// Pins auf Eingang setzen
  			DDRD &= ~(1 << DDD3);
  			PORTD &=~(1<<PD2);
  			PORTD &=~(1<<PD3);
  			set_USB2FC();

  			break;
  		case Uart02Wi:
  			set_Uart02Wi();
  			break;
  		case Uart02FC:
  			set_Uart02FC();
  			break;
  		case USB2Wi:
//  			lcd_printpns_at (0, 5, PSTR("set USB2Wi"),0);
//  			set_USB2Wi();
 			/* Disable UART receiver and transmitter */
//  			UCSR1B &= ~ ( ( 1 << RXCIE0 ) | ( 1 << RXEN0 ) | ( 1 << TXEN0 ) );
 // 			UCSR1B &= ~ _BV(RXCIE1)|(1<<RXEN1)|(1<<TXEN1); // Rx/Tx Disable
  			UCSR1B &= ~(1<<RXEN1 );
  			UCSR1B &= ~(1<<TXEN1 );
  			UCSR1B &= ~(1<<RXCIE1);

  			DDRD &= ~(1 << DDD2);					// Pins auf Eingang setzen
  			DDRD &= ~(1 << DDD3);
  			PORTD &=~(1<<PD2);						// Pullup aus
  			PORTD &=~(1<<PD3);
  			set_USB2Wi();
//  			UCSR1B = _BV(RXCIE1)|(1<<RXEN1)|(1<<TXEN1);
//  			DDRD &= ~(1 << DDD2);					// Pins auf Eingang setzen
//  			DDRD &= ~(1 << DDD3);
  			break;
  		case Uart12USB:
//  			lcd_printpns_at (0, 5, PSTR("set Uart12USB"),0);
  			//  		DO DOOOOO!!!!!!!!!!!!!!!!!	set_Uart12USB();		//Verbindung USB zu FC trennen
  			UCSR1B |= (1<<RXEN1 );
  			UCSR1B |=(1<<TXEN1 );
  			UCSR1B |=(1<<RXCIE1);

//  			uart1_init (UART_BAUD_SELECT(USART_BAUD,F_CPU));
//  			_delay_ms(2000);
  			break;
  		}

	}


/*************************************************************************
Function: BT2Wi()
Purpose:  Connect BT direct to Wi.232,
Returns:

**************************************************************************/
void Port_BT2Wi(void)
{

	lcd_cls ();
	lcd_printpns_at (0, 0, PSTR(" MK-USB Funktion     "), 2);
	lcd_printpns_at (0, 1, PSTR(" BT --> Wi.232       "), 2);
	lcd_printpns_at (0, 3, PSTR("PC mit BT verbinden"),0);
	lcd_printpns_at (0, 4, PSTR("Wi.232 an FC"),0);
	lcd_printpns_at (0, 5, PSTR("MK-Tool starten"),0);
	lcd_printpns_at (12, 7, PSTR("Esc"),0);

	set_BTOn();
	Change_Output(USB2Wi);
	for(;;)
	{
		if (get_key_press (1 << KEY_ESC))
		{
			get_key_press(KEY_ALL);

			if (U02SV2 == 1)
				Change_Output(Uart12USB);
			else
				Change_Output(Uart02Wi);

			set_USBOn();
			return;
		}
	}

}
/*************************************************************************
Function: BT2FC()
Purpose:  Connect BT direct to FC SV2 as MKUSB,
Returns:

**************************************************************************/
void Port_BT2FC(void)
{
	lcd_cls ();
	lcd_printpns_at (0, 0, PSTR(" MK-USB Funktion     "), 2);
	lcd_printpns_at (0, 1, PSTR(" BT --> Kabel an FC  "), 2);
	lcd_printpns_at (0, 3, PSTR("PC mit BT verb."),0);
	lcd_printpns_at (0, 4, PSTR("PKT-Kabel an FC"),0);
	lcd_printpns_at (0, 5, PSTR("MK-Tool starten"),0);
	lcd_printpns_at (12, 7, PSTR("Esc"),0);

	set_BTOn();
	Change_Output(USB2FC);
	for(;;)
	{
		if (get_key_press (1 << KEY_ESC))
		{
			get_key_press(KEY_ALL);

			if (U02SV2 == 1)
				Change_Output(Uart12USB);
			else
				Change_Output(Uart02Wi);

			set_USBOn();
			return;
		}
	}

}
/*************************************************************************
Function: USB2FC()
Purpose:  Connect USB direct to FC SV2 as MKUSB,
Returns:

**************************************************************************/
void Port_USB2FC(void)
{

	lcd_cls ();
	lcd_printpns_at (0, 0, PSTR("MK-USB Funktion"),0);
	lcd_printpns_at (0, 1, PSTR("USB <--> SV2(FC)"),0);
	lcd_printpns_at (0, 2, PSTR("PC mit USB/BT verb."),0);
	lcd_printpns_at (0, 3, PSTR("PKT SV2 Kabel an FC"),0);
	lcd_printpns_at (0, 4, PSTR("MK-Tool starten"),0);
	lcd_printpns_at (17, 7, PSTR("Exit"),0);
	Change_Output(USB2FC);
	 for(;;)
	    {
	        if ((get_key_press (1 << KEY_ENTER)))
	   		 {
	          Change_Output(Uart12USB);
	   	      return;
	   		 }
	    }

}
/*************************************************************************
Function: USB2Wi()
Purpose:  Connect USB direct to Wi.232,
Returns:

**************************************************************************/
void Port_USB2Wi(void)
{

	lcd_cls ();
	lcd_printpns_at (0, 0, PSTR("MK-USB Funktion"),0);
	lcd_printpns_at (0, 1, PSTR("USB --> Wi.232"),0);
	lcd_printpns_at (0, 2, PSTR("PC mit USB/BT verb."),0);
	lcd_printpns_at (0, 3, PSTR("Wi.232 an FC"),0);
	lcd_printpns_at (0, 4, PSTR("MK-Tool starten"),0);
	lcd_printpns_at (17, 7, PSTR("Exit"),0);
	Change_Output(USB2Wi);
	 for(;;)
      {
        if ((get_key_press (1 << KEY_ENTER)))
		 {
	      Change_Output(Uart12USB);
	      return;
	     }
	  }

}

/*************************************************************************
Function: USB2_CFGWi()
Purpose:  Connect USB direct to Wi.232 in Progmode,
Returns:

**************************************************************************/
void Port_USB2_CFGWi(void)
{

	lcd_cls ();
	lcd_printpns_at (0, 0, PSTR("Konfig Wi.232"),0);
	lcd_printpns_at (0, 1, PSTR("USB <--> Wi.232"),0);
	lcd_printpns_at (0, 2, PSTR("PC mit USB/BT verb."),0);
	lcd_printpns_at (0, 3, PSTR("Wi.232 Konfigprogramm"),0);
	lcd_printpns_at (0, 4, PSTR("starten"),0);
	lcd_printpns_at (17, 7, PSTR("Exit"),0);
	Change_Output(USB2Wi);
	set_WI232CMD();
	 for(;;)
      {
        if ((get_key_press (1 << KEY_ENTER)))
		 {
          clr_WI232CMD();
	      Change_Output(Uart12USB);
	      return;
	     }
	  }

}


#endif
#endif /* HAL_HW3_9_C_ */
